-- C97201C.ADA

--                             Grant of Unlimited Rights
--
--     Under contracts F33600-87-D-0337, F33600-84-D-0280, MDA903-79-C-0687,
--     F08630-91-C-0015, and DCA100-97-D-0025, the U.S. Government obtained 
--     unlimited rights in the software and documentation contained herein.
--     Unlimited rights are defined in DFAR 252.227-7013(a)(19).  By making 
--     this public release, the Government intends to confer upon all 
--     recipients unlimited rights  equal to those held by the Government.  
--     These rights include rights to use, duplicate, release or disclose the 
--     released technical data and computer software in whole or in part, in 
--     any manner and for any purpose whatsoever, and to have or permit others 
--     to do so.
--
--                                    DISCLAIMER
--
--     ALL MATERIALS OR INFORMATION HEREIN RELEASED, MADE AVAILABLE OR
--     DISCLOSED ARE AS IS.  THE GOVERNMENT MAKES NO EXPRESS OR IMPLIED 
--     WARRANTY AS TO ANY MATTER WHATSOEVER, INCLUDING THE CONDITIONS OF THE
--     SOFTWARE, DOCUMENTATION OR OTHER INFORMATION RELEASED, MADE AVAILABLE 
--     OR DISCLOSED, OR THE OWNERSHIP, MERCHANTABILITY, OR FITNESS FOR A
--     PARTICULAR PURPOSE OF SAID MATERIAL.
--*
-- CHECK THAT A CONDITIONAL ENTRY CALL IS NOT ACCEPTED IF AN ACCEPT
-- STATEMENT FOR THE CALLED ENTRY HAS NOT YET BEEN REACHED.

-- WRG 7/11/86

WITH REPORT; USE REPORT;
PROCEDURE C97201C IS

BEGIN

     TEST ("C97201C", "CHECK THAT A CONDITIONAL ENTRY CALL IS NOT " &
                      "ACCEPTED IF AN ACCEPT STATEMENT FOR THE " &
                      "CALLED ENTRY HAS NOT YET BEEN REACHED");

     DECLARE

          TASK T IS
               ENTRY E;
               ENTRY BARRIER;
          END T;

          TASK BODY T IS
          BEGIN
               ACCEPT BARRIER;
               IF E'COUNT > 0 THEN
                    FAILED ("ENTRY CALL WAS QUEUED");
                    ACCEPT E;
               END IF;
          END T;

     BEGIN

          SELECT
               T.E;
               FAILED ("CONDITIONAL ENTRY CALL ACCEPTED");
          ELSE
               COMMENT ("ELSE PART EXECUTED");
          END SELECT;

          T.BARRIER;

     END;

     RESULT;

END C97201C;
